@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fdlimit.c 5689 2008-04-25 02:57:33Z charles $
+ * $Id: fdlimit.c 7051 2008-11-05 05:56:06Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,7 +23,7 @@
  *****************************************************************************/
 
 #ifndef WIN32
-#define HAVE_GETRLIMIT
+ #define HAVE_GETRLIMIT
 #endif
 
 #include <assert.h>
@@ -32,34 +32,34 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef SYS_DARWIN
+#include <fcntl.h>
+#endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef HAVE_GETRLIMIT
-#include <sys/time.h> /* getrlimit */
-#include <sys/resource.h> /* getrlimit */
+ #include <sys/time.h> /* getrlimit */
+ #include <sys/resource.h> /* getrlimit */
 #endif
 #include <unistd.h>
-#include <libgen.h> /* dirname */
 #include <fcntl.h> /* O_LARGEFILE */
 
 #include <event.h>
 #include <evutil.h>
 
 #include "transmission.h"
+#include "fdlimit.h"
 #include "list.h"
 #include "net.h"
 #include "platform.h" /* tr_lock */
-#include "trcompat.h" /* strlcpy */
 #include "utils.h"
 
-#if SIZEOF_VOIDP==8
-#define TR_UINT_TO_PTR(i) (void*)((uint64_t)i)
-#else
-#define TR_UINT_TO_PTR(i) ((void*)((uint32_t)i))
-#endif
-
-#define dbgmsg(fmt...) tr_deepLog( __FILE__, __LINE__, NULL, ##fmt )
+#define dbgmsg( ... ) \
+    do { \
+        if( tr_deepLoggingIsActive( ) ) \
+            tr_deepLog( __FILE__, __LINE__, NULL, __VA_ARGS__ ); \
+    } while( 0 )
 
 /**
 ***
@@ -75,21 +75,20 @@ enum
 
 struct tr_openfile
 {
-    unsigned int  isCheckedOut : 1;
-    unsigned int  isWritable : 1;
-    unsigned int  closeWhenDone : 1;
-    char          filename[MAX_PATH_LENGTH];
-    int           fd;
-    uint64_t      date;
+    unsigned int    isCheckedOut  : 1;
+    unsigned int    isWritable    : 1;
+    unsigned int    closeWhenDone : 1;
+    char            filename[MAX_PATH_LENGTH];
+    int             fd;
+    uint64_t        date;
 };
 
 struct tr_fd_s
 {
-    int                  reserved;
-    int                  normal;
-    int                  normalMax;
-    tr_lock            * lock;
-    struct tr_openfile   open[TR_MAX_OPEN_FILES];
+    int                   socketCount;
+    int                   socketMax;
+    tr_lock *             lock;
+    struct tr_openfile    open[TR_MAX_OPEN_FILES];
 };
 
 static struct tr_fd_s * gFd = NULL;
@@ -100,33 +99,75 @@ static struct tr_fd_s * gFd = NULL;
 ****
 ***/
 
-static tr_errno
-TrOpenFile( int           i,
-            const char  * folder,
-            const char  * torrentFile,
-            int           write )
+static int
+preallocateFile( int fd UNUSED, uint64_t length UNUSED )
+{
+#ifdef HAVE_FALLOCATE
+
+    return fallocate( fd, 0, offset, length );
+
+#elif defined(HAVE_POSIX_FALLOCATE)
+
+    return posix_fallocate( fd, 0, length );
+
+#elif defined(SYS_DARWIN) 
+
+    fstore_t fst;
+    fst.fst_flags = F_ALLOCATECONTIG;
+    fst.fst_posmode = F_PEOFPOSMODE;
+    fst.fst_offset = 0;
+    fst.fst_length = length;
+    fst.fst_bytesalloc = 0;
+    return fcntl( fd, F_PREALLOCATE, &fst );
+
+#else
+
+    #warning no known method to preallocate files on this platform
+    return -1;
+
+#endif
+}
+
+/**
+ * returns 0 on success, or an errno value on failure.
+ * errno values include ENOENT if the parent folder doesn't exist,
+ * plus the errno values set by tr_mkdirp() and open().
+ */
+static int
+TrOpenFile( int          i,
+            const char * folder,
+            const char * torrentFile,
+            int          doWrite,
+            int          doPreallocate,
+            uint64_t     desiredFileSize )
 {
     struct tr_openfile * file = &gFd->open[i];
-    int flags;
-    char filename[MAX_PATH_LENGTH];
-    struct stat sb;
+    int                  flags;
+    char               * filename;
+    struct stat          sb;
+    int                  alreadyExisted;
 
     /* confirm the parent folder exists */
     if( stat( folder, &sb ) || !S_ISDIR( sb.st_mode ) )
-        return TR_ERROR_IO_PARENT;
+        return ENOENT;
 
     /* create subfolders, if any */
-    tr_buildPath ( filename, sizeof(filename), folder, torrentFile, NULL );
-    if( write ) {
-        char * tmp = tr_strdup( filename );
-        const int err = tr_mkdirp( dirname(tmp), 0777 ) ? errno : 0;
+    filename = tr_buildPath( folder, torrentFile, NULL );
+    if( doWrite )
+    {
+        char * tmp = tr_dirname( filename );
+        const int err = tr_mkdirp( tmp, 0777 ) ? errno : 0;
         tr_free( tmp );
-        if( err )
-            return tr_ioErrorFromErrno( err );
+        if( err ) {
+            tr_free( filename );
+            return err;
+        }
     }
 
+    alreadyExisted = !stat( filename, &sb ) && S_ISREG( sb.st_mode );
+    
     /* open the file */
-    flags = write ? (O_RDWR | O_CREAT) : O_RDONLY;
+    flags = doWrite ? ( O_RDWR | O_CREAT ) : O_RDONLY;
 #ifdef O_LARGEFILE
     flags |= O_LARGEFILE;
 #endif
@@ -134,13 +175,21 @@ TrOpenFile( int           i,
     flags |= O_BINARY;
 #endif
     file->fd = open( filename, flags, 0666 );
-    if( file->fd == -1 ) {
+    if( file->fd == -1 )
+    {
         const int err = errno;
-        tr_err( _( "Couldn't open \"%1$s\": %2$s" ), filename, tr_strerror(err) );
-        return tr_ioErrorFromErrno( err );
+        tr_err( _( "Couldn't open \"%1$s\": %2$s" ), filename,
+               tr_strerror( err ) );
+        tr_free( filename );
+        return err;
     }
 
-    return TR_OK;
+    if( ( file->fd >= 0 ) && !alreadyExisted && doPreallocate )
+        if( !preallocateFile( file->fd, desiredFileSize ) )
+            tr_inf( _( "Preallocated file \"%s\"" ), filename );
+       
+    tr_free( filename );
+    return 0;
 }
 
 static int
@@ -166,29 +215,33 @@ TrCloseFile( int i )
 static int
 fileIsCheckedOut( const struct tr_openfile * o )
 {
-    return fileIsOpen(o) && o->isCheckedOut;
+    return fileIsOpen( o ) && o->isCheckedOut;
 }
 
+/* returns an fd on success, or a -1 on failure and sets errno */
 int
 tr_fdFileCheckout( const char * folder,
-                   const char * torrentFile, 
-                   int          write )
+                   const char * torrentFile,
+                   int          doWrite,
+                   int          doPreallocate,
+                   uint64_t     desiredFileSize )
 {
-    int i, winner = -1;
+    int                  i, winner = -1;
     struct tr_openfile * o;
-    char filename[MAX_PATH_LENGTH];
+    char               * filename;
 
     assert( folder && *folder );
     assert( torrentFile && *torrentFile );
-    assert( write==0 || write==1 );
+    assert( doWrite == 0 || doWrite == 1 );
 
-    tr_buildPath ( filename, sizeof(filename), folder, torrentFile, NULL );
-    dbgmsg( "looking for file '%s', writable %c", filename, write?'y':'n' );
+    filename = tr_buildPath( folder, torrentFile, NULL );
+    dbgmsg( "looking for file '%s', writable %c", filename,
+            doWrite ? 'y' : 'n' );
 
     tr_lockLock( gFd->lock );
 
     /* Is it already open? */
-    for( i=0; i<TR_MAX_OPEN_FILES; ++i )
+    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
     {
         o = &gFd->open[i];
 
@@ -198,7 +251,8 @@ tr_fdFileCheckout( const char * folder,
         if( strcmp( filename, o->filename ) )
             continue;
 
-        if( fileIsCheckedOut( o ) ) {
+        if( fileIsCheckedOut( o ) )
+        {
             dbgmsg( "found it!  it's open, but checked out.  waiting..." );
             tr_lockUnlock( gFd->lock );
             tr_wait( 200 );
@@ -207,8 +261,10 @@ tr_fdFileCheckout( const char * folder,
             continue;
         }
 
-        if( write && !o->isWritable ) {
-            dbgmsg( "found it!  it's open and available, but isn't writable. closing..." );
+        if( doWrite && !o->isWritable )
+        {
+            dbgmsg(
+                "found it!  it's open and available, but isn't writable. closing..." );
             TrCloseFile( i );
             break;
         }
@@ -218,35 +274,45 @@ tr_fdFileCheckout( const char * folder,
         break;
     }
 
-    dbgmsg( "it's not already open.  looking for an open slot or an old file." );
+    dbgmsg(
+        "it's not already open.  looking for an open slot or an old file." );
     while( winner < 0 )
     {
         uint64_t date = tr_date( ) + 1;
 
         /* look for the file that's been open longest */
-        for( i=0; i<TR_MAX_OPEN_FILES; ++i )
+        for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
         {
             o = &gFd->open[i];
 
-            if( !fileIsOpen( o ) ) {
+            if( !fileIsOpen( o ) )
+            {
                 winner = i;
                 dbgmsg( "found an empty slot in %d", winner );
                 break;
             }
 
-            if( date > o->date ) {
+            if( date > o->date )
+            {
                 date = o->date;
                 winner = i;
             }
         }
 
-        if( winner >= 0 ) {
-            if( fileIsOpen( &gFd->open[winner] ) ) {
-                dbgmsg( "closing file '%s', slot #%d", gFd->open[winner].filename, winner );
+        if( winner >= 0 )
+        {
+            if( fileIsOpen( &gFd->open[winner] ) )
+            {
+                dbgmsg( "closing file '%s', slot #%d",
+                        gFd->open[winner].filename,
+                        winner );
                 TrCloseFile( winner );
             }
-        } else { 
-            dbgmsg( "everything's full!  waiting for someone else to finish something" );
+        }
+        else
+        {
+            dbgmsg(
+                "everything's full!  waiting for someone else to finish something" );
             tr_lockUnlock( gFd->lock );
             tr_wait( 200 );
             tr_lockLock( gFd->lock );
@@ -257,21 +323,25 @@ tr_fdFileCheckout( const char * folder,
     o = &gFd->open[winner];
     if( !fileIsOpen( o ) )
     {
-        const tr_errno err = TrOpenFile( winner, folder, torrentFile, write );
+        const int err = TrOpenFile( winner, folder, torrentFile, doWrite, doPreallocate, desiredFileSize );
         if( err ) {
             tr_lockUnlock( gFd->lock );
-            return err;
+            tr_free( filename );
+            errno = err;
+            return -1;
         }
 
-        dbgmsg( "opened '%s' in slot %d, write %c", filename, winner, write?'y':'n' );
-        strlcpy( o->filename, filename, sizeof( o->filename ) );
-        o->isWritable = write;
+        dbgmsg( "opened '%s' in slot %d, doWrite %c", filename, winner,
+                doWrite ? 'y' : 'n' );
+        tr_strlcpy( o->filename, filename, sizeof( o->filename ) );
+        o->isWritable = doWrite;
     }
 
     dbgmsg( "checking out '%s' in slot %d", filename, winner );
     o->isCheckedOut = 1;
     o->closeWhenDone = 0;
     o->date = tr_date( );
+    tr_free( filename );
     tr_lockUnlock( gFd->lock );
     return o->fd;
 }
@@ -280,9 +350,10 @@ void
 tr_fdFileReturn( int fd )
 {
     int i;
+
     tr_lockLock( gFd->lock );
 
-    for( i=0; i<TR_MAX_OPEN_FILES; ++i )
+    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
     {
         struct tr_openfile * o = &gFd->open[i];
         if( o->fd != fd )
@@ -292,10 +363,10 @@ tr_fdFileReturn( int fd )
         o->isCheckedOut = 0;
         if( o->closeWhenDone )
             TrCloseFile( i );
-        
+
         break;
     }
-    
+
     tr_lockUnlock( gFd->lock );
 }
 
@@ -303,25 +374,31 @@ void
 tr_fdFileClose( const char * filename )
 {
     int i;
+
     tr_lockLock( gFd->lock );
 
-    for( i=0; i<TR_MAX_OPEN_FILES; ++i )
+    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
     {
         struct tr_openfile * o = &gFd->open[i];
-        if( !fileIsOpen(o) || strcmp(filename,o->filename) )
+        if( !fileIsOpen( o ) || strcmp( filename, o->filename ) )
             continue;
 
         dbgmsg( "tr_fdFileClose closing '%s'", filename );
 
-        if( !o->isCheckedOut ) {
+        if( !o->isCheckedOut )
+        {
             dbgmsg( "not checked out, so closing it now... '%s'", filename );
             TrCloseFile( i );
-        } else {
-            dbgmsg( "flagging file '%s', slot #%d to be closed when checked in", gFd->open[i].filename, i );
+        }
+        else
+        {
+            dbgmsg(
+                "flagging file '%s', slot #%d to be closed when checked in",
+                gFd->open[i].filename, i );
             o->closeWhenDone = 1;
         }
     }
-    
+
     tr_lockUnlock( gFd->lock );
 }
 
@@ -331,76 +408,56 @@ tr_fdFileClose( const char * filename )
 ****
 ***/
 
-static tr_list * reservedSockets = NULL;
-
-static void
-setSocketPriority( int fd, int isReserved )
-{
-    if( isReserved )
-        tr_list_append( &reservedSockets, TR_UINT_TO_PTR(fd) );
-}
-
-static int
-socketWasReserved( int fd )
-{
-    return tr_list_remove_data( &reservedSockets, TR_UINT_TO_PTR(fd) ) != NULL;
-}
-
 static int
 getSocketMax( struct tr_fd_s * gFd )
 {
-    return gFd->normalMax;
+    return gFd->socketMax;
 }
 
 int
-tr_fdSocketCreate( int type, int isReserved )
+tr_fdSocketCreate( int type )
 {
     int s = -1;
+
     tr_lockLock( gFd->lock );
 
-    if( isReserved || ( gFd->normal < getSocketMax( gFd ) ) )
+    if( gFd->socketCount < getSocketMax( gFd ) )
         if( ( s = socket( AF_INET, type, 0 ) ) < 0 )
-            tr_err( _( "Couldn't create socket: %s" ), tr_strerror( sockerrno ) );
+            tr_err( _( "Couldn't create socket: %s" ),
+                   tr_strerror( sockerrno ) );
 
     if( s > -1 )
-    {
-        setSocketPriority( s, isReserved );
+        ++gFd->socketCount;
 
-        if( isReserved )
-            ++gFd->reserved;
-        else
-            ++gFd->normal;
-    }
-
-    assert( gFd->reserved >= 0 );
-    assert( gFd->normal >= 0 );
+    assert( gFd->socketCount >= 0 );
 
     tr_lockUnlock( gFd->lock );
     return s;
 }
 
 int
-tr_fdSocketAccept( int b, struct in_addr * addr, tr_port_t * port )
+tr_fdSocketAccept( int              b,
+                   struct in_addr * addr,
+                   tr_port_t *      port )
 {
-    int s = -1;
-    unsigned int len;
+    int                s = -1;
+    unsigned int       len;
     struct sockaddr_in sock;
 
-    assert( addr != NULL );
-    assert( port != NULL );
+    assert( addr );
+    assert( port );
 
     tr_lockLock( gFd->lock );
-    if( gFd->normal < getSocketMax( gFd ) )
+    if( gFd->socketCount < getSocketMax( gFd ) )
     {
         len = sizeof( sock );
         s = accept( b, (struct sockaddr *) &sock, &len );
     }
     if( s > -1 )
     {
-        setSocketPriority( s, FALSE );
         *addr = sock.sin_addr;
         *port = sock.sin_port;
-        gFd->normal++;
+        ++gFd->socketCount;
     }
     tr_lockUnlock( gFd->lock );
 
@@ -422,16 +479,13 @@ tr_fdSocketClose( int s )
 {
     tr_lockLock( gFd->lock );
 
-    if( s >= 0 ) {
+    if( s >= 0 )
+    {
         socketClose( s );
-        if( socketWasReserved( s ) )
-            --gFd->reserved;
-        else
-            --gFd->normal;
+        --gFd->socketCount;
     }
 
-    assert( gFd->reserved >= 0 );
-    assert( gFd->normal >= 0 );
+    assert( gFd->socketCount >= 0 );
 
     tr_lockUnlock( gFd->lock );
 }
@@ -456,17 +510,17 @@ tr_fdInit( int globalPeerLimit )
         struct rlimit rlim;
         getrlimit( RLIMIT_NOFILE, &rlim );
         rlim.rlim_cur = MIN( rlim.rlim_max,
-               (rlim_t)(globalPeerLimit + NOFILE_BUFFER) );
+                            (rlim_t)( globalPeerLimit + NOFILE_BUFFER ) );
         setrlimit( RLIMIT_NOFILE, &rlim );
-        gFd->normalMax = rlim.rlim_cur - NOFILE_BUFFER;
+        gFd->socketMax = rlim.rlim_cur - NOFILE_BUFFER;
         tr_dbg( "setrlimit( RLIMIT_NOFILE, %d )", (int)rlim.rlim_cur );
     }
 #else
-    gFd->normalMax = globalPeerLimit;
+    gFd->socketMax = globalPeerLimit;
 #endif
     tr_dbg( "%d usable file descriptors", globalPeerLimit );
 
-    for( i=0; i<TR_MAX_OPEN_FILES; ++i ) 
+    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
         gFd->open[i].fd = -1;
 }
 
@@ -475,25 +529,26 @@ tr_fdClose( void )
 {
     int i = 0;
 
-    for( i=0; i<TR_MAX_OPEN_FILES; ++i )
+    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
         if( fileIsOpen( &gFd->open[i] ) )
             TrCloseFile( i );
 
     tr_lockFree( gFd->lock );
 
-    tr_list_free( &reservedSockets, NULL );
     tr_free( gFd );
+    gFd = NULL;
 }
 
 void
 tr_fdSetPeerLimit( uint16_t n )
 {
-    assert( gFd!=NULL && "tr_fdInit() must be called first!" );
-    gFd->normalMax = n;
+    assert( gFd != NULL && "tr_fdInit() must be called first!" );
+    gFd->socketMax = n;
 }
 
 uint16_t
 tr_fdGetPeerLimit( void )
 {
-    return gFd ? gFd->normalMax : -1;
+    return gFd ? gFd->socketMax : -1;
 }
+
