@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: BadgeView.h 5687 2008-04-24 21:24:42Z livings124 $
+ * $Id: BadgeView.h 6959 2008-10-25 21:34:30Z livings124 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -31,13 +31,13 @@
     
     NSDictionary * fAttributes;
     
-    float fDownloadRate, fUploadRate;
+    CGFloat fDownloadRate, fUploadRate;
     BOOL fQuitting;
 }
 
 - (id) initWithFrame: (NSRect) frame lib: (tr_handle *) lib;
 
-- (BOOL) setRatesWithDownload: (float) downloadRate upload: (float) uploadRate;
+- (BOOL) setRatesWithDownload: (CGFloat) downloadRate upload: (CGFloat) uploadRate;
 - (void) setQuitting;
 
 @end
