@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Controller.h 5703 2008-04-25 22:23:31Z livings124 $
+ * $Id: Controller.h 7017 2008-11-02 13:52:45Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -30,18 +30,18 @@
 #import "AddWindowController.h"
 #import "DragOverlayWindow.h"
 #import "Badger.h"
-#import "StatusBarView.h"
-#import "FilterButton.h"
-#import "MenuLabel.h"
-#import "IPCController.h"
 
 #import <Growl/Growl.h>
 
 @class TorrentTableView;
+@class StatusBarView;
+@class FilterButton;
+@class MenuLabel;
 
 typedef enum
 {
-    ADD_NORMAL,
+    ADD_MANUAL,
+    ADD_AUTO,
     ADD_SHOW_OPTIONS,
     ADD_URL,
     ADD_CREATED
@@ -56,20 +56,20 @@ typedef enum
     PrefsController                 * fPrefsController;
     InfoWindowController            * fInfoController;
     MessageWindowController         * fMessageController;
-    IPCController                   * fIPCController;
     
     NSUserDefaults                  * fDefaults;
     
     IBOutlet NSWindow               * fWindow;
     DragOverlayWindow               * fOverlayWindow;
     IBOutlet TorrentTableView       * fTableView;
+
+    io_connect_t                    fRootPort;
+    NSTimer                         * fTimer;
     
     IBOutlet NSMenuItem             * fOpenIgnoreDownloadFolder;
     
-    IBOutlet StatusBarView          * fBottomTigerBar;
     IBOutlet NSBox                  * fBottomTigerLine;
     IBOutlet NSButton               * fActionButton, * fSpeedLimitButton;
-    NSTimer                         * fSpeedLimitTimer;
     IBOutlet NSTextField            * fTotalTorrentsField;
     
     IBOutlet StatusBarView          * fStatusBar;
@@ -100,25 +100,20 @@ typedef enum
     IBOutlet NSWindow               * fURLSheetWindow;
     IBOutlet NSTextField            * fURLSheetTextField;
     IBOutlet NSButton               * fURLSheetOpenButton;
-
-    io_connect_t                    fRootPort;
-    NSTimer                         * fTimer;
     
-    IBOutlet SUUpdater              * fUpdater;
     BOOL                            fUpdateInProgress;
     
     Badger                          * fBadger;
     IBOutlet NSMenu                 * fDockMenu;
     
+    NSTimer                         * fSpeedLimitTimer;
+    
     NSMutableArray                  * fAutoImportedNames;
     NSTimer                         * fAutoImportTimer;
     
     NSMutableDictionary             * fPendingTorrentDownloads;
-    NSMutableArray                  * fTempTorrentFiles;
     
     BOOL                            fSoundPlaying;
-    
-    BOOL                            fRemoteQuit;
 }
 
 - (void) openFiles:             (NSArray *) filenames addType: (addType) type forcePath: (NSString *) path;
@@ -127,6 +122,7 @@ typedef enum
 - (void) openFilesWithDict:     (NSDictionary *) dictionary;
 - (void) openShowSheet:         (id) sender;
 
+- (void) invalidOpenAlert: (NSString *) filename;
 - (void) duplicateOpenAlert: (NSString *) name;
 
 - (void) openURL:               (NSURL *) torrentURL;
@@ -134,7 +130,7 @@ typedef enum
 - (void) openURLCancelEndSheet: (id) sender;
 - (void) openURLShowSheet:      (id) sender;
 
-- (void) quitSheetDidEnd: (NSWindow *) sheet returnCode: (int) returnCode contextInfo: (void *) contextInfo;
+- (void) quitSheetDidEnd: (NSWindow *) sheet returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo;
 
 - (void) createFile: (id) sender;
 
@@ -152,10 +148,9 @@ typedef enum
 
 - (void) removeTorrents: (NSArray *) torrents
         deleteData: (BOOL) deleteData deleteTorrent: (BOOL) deleteData;
-- (void) removeSheetDidEnd: (NSWindow *) sheet returnCode: (int) returnCode
+- (void) removeSheetDidEnd: (NSWindow *) sheet returnCode: (NSInteger) returnCode
                         contextInfo: (NSDictionary *) dict;
-- (void) confirmRemoveTorrents: (NSArray *) torrents
-        deleteData: (BOOL) deleteData deleteTorrent: (BOOL) deleteTorrent;
+- (void) confirmRemoveTorrents: (NSArray *) torrents deleteData: (BOOL) deleteData deleteTorrent: (BOOL) deleteTorrent;
 - (void) removeNoDelete:                (id) sender;
 - (void) removeDeleteData:              (id) sender;
 - (void) removeDeleteTorrent:           (id) sender;
@@ -163,7 +158,7 @@ typedef enum
 
 - (void) moveDataFilesSelected: (id) sender;
 - (void) moveDataFiles: (NSArray *) torrents;
-- (void) moveDataFileChoiceClosed: (NSOpenPanel *) panel returnCode: (int) code contextInfo: (NSArray *) torrents;
+- (void) moveDataFileChoiceClosed: (NSOpenPanel *) panel returnCode: (NSInteger) code contextInfo: (NSArray *) torrents;
 
 - (void) copyTorrentFiles: (id) sender;
 - (void) copyTorrentFileForTorrents: (NSMutableArray *) torrents;
@@ -180,6 +175,7 @@ typedef enum
 - (void) showAboutWindow: (id) sender;
 
 - (void) showInfo: (id) sender;
+- (void) resetInfo;
 - (void) setInfoTab: (id) sender;
 
 - (void) showMessageWindow: (id) sender;
@@ -191,7 +187,7 @@ typedef enum
 - (void) setBottomCountText: (BOOL) filtering;
 
 - (void) updateTorrentsInQueue;
-- (int) numToStartFromQueue: (BOOL) downloadQueue;
+- (NSInteger) numToStartFromQueue: (BOOL) downloadQueue;
 
 - (void) torrentFinishedDownloading: (NSNotification *) notification;
 - (void) torrentRestartedDownloading: (NSNotification *) notification;
@@ -221,7 +217,8 @@ typedef enum
 
 - (void) toggleSpeedLimit: (id) sender;
 - (void) autoSpeedLimitChange: (NSNotification *) notification;
-- (void) autoSpeedLimit;
+- (void) autoSpeedLimit: (NSTimer *) timer;
+- (void) setAutoSpeedLimitTimer: (BOOL) nextIsLimit;
 
 - (void) setLimitGlobalEnabled: (id) sender;
 - (void) setQuickLimitGlobal: (id) sender;
@@ -234,13 +231,14 @@ typedef enum
 
 - (void) beginCreateFile: (NSNotification *) notification;
 
-- (void) sleepCallBack: (natural_t) messageType argument: (void *) messageArgument;
+- (void) sleepCallback: (natural_t) messageType argument: (void *) messageArgument;
 
 - (void) torrentTableViewSelectionDidChange: (NSNotification *) notification;
 
 - (void) toggleSmallView: (id) sender;
 - (void) togglePiecesBar: (id) sender;
 - (void) toggleAvailabilityBar: (id) sender;
+- (void) toggleStatusString: (id) sender;
 
 - (void) toggleStatusBar: (id) sender;
 - (void) showStatusBar: (BOOL) show animate: (BOOL) animate;
@@ -258,10 +256,21 @@ typedef enum
 
 - (void) showMainWindow: (id) sender;
 
+- (NSArray *) quickLookURLs;
+- (BOOL) canQuickLook;
+- (BOOL) canQuickLookTorrent: (Torrent *) torrent;
+- (NSRect) quickLookFrameWithURL: (NSURL*) url;
+- (void) toggleQuickLook: (id) sender;
+
 - (void) linkHomepage: (id) sender;
 - (void) linkForums: (id) sender;
+- (void) linkTrac: (id) sender;
 - (void) linkDonate: (id) sender;
 
-- (void) prepareForUpdate:  (NSNotification *) notification;
+- (void) rpcCallback: (tr_rpc_callback_type) type forTorrentStruct: (struct tr_torrent *) torrentStruct;
+- (void) rpcAddTorrentStruct: (NSValue *) torrentStructPtr;
+- (void) rpcRemoveTorrent: (Torrent *) torrent;
+- (void) rpcStartedStoppedTorrent: (Torrent *) torrent;
+- (void) rpcChangedTorrent: (Torrent *) torrent;
 
 @end
