@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: CreatorWindowController.m 5297 2008-03-18 22:11:43Z livings124 $
+ * $Id: CreatorWindowController.m 7032 2008-11-04 01:31:24Z livings124 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -23,14 +23,23 @@
  *****************************************************************************/
 
 #import "CreatorWindowController.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
+#import "utils.h" //tr_httpIsValidURL
+
+#define TRACKER_ADD_TAG 0
+#define TRACKER_REMOVE_TAG 1
 
 @interface CreatorWindowController (Private)
 
 + (NSString *) chooseFile;
-- (void) locationSheetClosed: (NSSavePanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
+- (void) updateEnableOpenCheckForTrackers;
+- (void) locationSheetClosed: (NSSavePanel *) openPanel returnCode: (NSInteger) code contextInfo: (void *) info;
+
+- (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo;
+- (void) createReal;
 - (void) checkProgress;
-- (void) failureSheetClosed: (NSAlert *) alert returnCode: (int) code contextInfo: (void *) info;
+- (void) failureSheetClosed: (NSAlert *) alert returnCode: (NSInteger) code contextInfo: (void *) info;
 
 @end
 
@@ -96,6 +105,31 @@
         }
         
         fDefaults = [NSUserDefaults standardUserDefaults];
+        
+        //get list of trackers
+        if (!(fTrackers = [[fDefaults arrayForKey: @"CreatorTrackers"] mutableCopy]))
+        {
+            fTrackers = [[NSMutableArray alloc] initWithCapacity: 1];
+            
+            //check for single tracker from versions before 1.3
+            NSString * tracker;
+            if ((tracker = [fDefaults stringForKey: @"CreatorTracker"]))
+            {
+                [fDefaults removeObjectForKey: @"CreatorTracker"];
+                if (![tracker isEqualToString: @""])
+                {
+                    [fTrackers addObject: tracker];
+                    [fDefaults setObject: fTrackers forKey: @"CreatorTrackers"];
+                }
+            }
+        }
+        
+        //remove potentially invalid addresses
+        for (NSInteger i = [fTrackers count]-1; i >= 0; i--)
+        {
+            if (!tr_httpIsValidURL([[fTrackers objectAtIndex: i] UTF8String]))
+                [fTrackers removeObjectAtIndex: i];
+        }
     }
     return self;
 }
@@ -120,11 +154,11 @@
     if (multifile)
     {
         NSString * fileString;
-        int count = fInfo->fileCount;
+        NSInteger count = fInfo->fileCount;
         if (count != 1)
-            fileString = [NSString stringWithFormat: NSLocalizedString(@"%d Files", "Create torrent -> info"), count];
+            fileString = [NSString stringWithFormat: NSLocalizedString(@"%d files", "Create torrent -> info"), count];
         else
-            fileString = NSLocalizedString(@"1 File", "Create torrent -> info");
+            fileString = NSLocalizedString(@"1 file", "Create torrent -> info");
         statusString = [NSString stringWithFormat: @"%@, %@", fileString, statusString];
     }
     [fStatusField setStringValue: statusString];
@@ -145,15 +179,18 @@
     [fLocationField setToolTip: fLocation];
     
     //set previously saved values
-    NSString * tracker;
-    if ((tracker = [fDefaults stringForKey: @"CreatorTracker"]))
-        [fTrackerField setStringValue: tracker];
-    
     if ([fDefaults objectForKey: @"CreatorPrivate"])
         [fPrivateCheck setState: [fDefaults boolForKey: @"CreatorPrivate"] ? NSOnState : NSOffState];
     
-    if ([fDefaults objectForKey: @"CreatorOpen"])
-        [fOpenCheck setState: [fDefaults boolForKey: @"CreatorOpen"] ? NSOnState : NSOffState];
+    fOpenTorrent = [fDefaults boolForKey: @"CreatorOpen"];
+    [self updateEnableOpenCheckForTrackers];
+    
+    if (![NSApp isOnLeopardOrBetter])
+    {
+        [fTrackerAddRemoveControl sizeToFit];
+        [fTrackerAddRemoveControl setLabel: @"+" forSegment: TRACKER_ADD_TAG];
+        [fTrackerAddRemoveControl setLabel: @"-" forSegment: TRACKER_REMOVE_TAG];
+    }
 }
 
 - (void) dealloc
@@ -161,15 +198,21 @@
     [fPath release];
     [fLocation release];
     
+    [fTrackers release];
+    
     if (fInfo)
         tr_metaInfoBuilderFree(fInfo);
     
-    if (fTimer)
-        [fTimer invalidate];
+    [fTimer invalidate];
     
     [super dealloc];
 }
 
+- (void) toggleOpenCheck: (id) sender
+{
+    fOpenTorrent = [fOpenCheck state] == NSOnState;
+}
+
 - (void) setLocation: (id) sender
 {
     NSSavePanel * panel = [NSSavePanel savePanel];
@@ -188,77 +231,26 @@
 
 - (void) create: (id) sender
 {
-    //parse tracker string
-    NSString * trackerString = [fTrackerField stringValue];
-    if ([trackerString rangeOfString: @"://"].location != NSNotFound)
+    if ([fTrackers count] == 0 && [fDefaults boolForKey: @"WarningCreatorBlankAddress"])
     {
-        if (![trackerString hasPrefix: @"http://"])
-        {
-            NSAlert * alert = [[[NSAlert alloc] init] autorelease];
-            [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> http warning -> button")];
-            [alert setMessageText: NSLocalizedString(@"The tracker address must begin with \"http://\".",
-                                                    "Create torrent -> http warning -> title")];
-            [alert setInformativeText: NSLocalizedString(@"Change the tracker address to create the torrent.",
-                                                        "Create torrent -> http warning -> warning")];
-            [alert setAlertStyle: NSWarningAlertStyle];
-            
-            [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
-            return;
-        }
+        NSAlert * alert = [[NSAlert alloc] init];
+        [alert setMessageText: NSLocalizedString(@"There is no tracker address.", "Create torrent -> blank address -> title")];
+        [alert setInformativeText: NSLocalizedString(@"The torrent file will not be able to be opened."
+            " A torrent file with no tracker address is only useful when you plan to upload the file to a tracker website"
+            " that will add the address for you.", "Create torrent -> blank address -> message")];
+        [alert addButtonWithTitle: NSLocalizedString(@"Create", "Create torrent -> blank address -> button")];
+        [alert addButtonWithTitle: NSLocalizedString(@"Cancel", "Create torrent -> blank address -> button")];
+        
+        if ([NSApp isOnLeopardOrBetter])
+            [alert setShowsSuppressionButton: YES];
+        else
+            [alert addButtonWithTitle: NSLocalizedString(@"Don't Alert Again", "Create torrent -> blank address -> button")];
+
+        [alert beginSheetModalForWindow: [self window] modalDelegate: self
+            didEndSelector: @selector(createBlankAddressAlertDidEnd:returnCode:contextInfo:) contextInfo: nil];
     }
     else
-        trackerString = [@"http://" stringByAppendingString: trackerString];
-    
-    //don't allow blank addresses
-    if ([trackerString length] <= 7)
-    {
-        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
-        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> no url warning -> button")];
-        [alert setMessageText: NSLocalizedString(@"The tracker address cannot be blank.",
-                                                "Create torrent -> no url warning -> title")];
-        [alert setInformativeText: NSLocalizedString(@"Change the tracker address to create the torrent.",
-                                                    "Create torrent -> no url warning -> warning")];
-        [alert setAlertStyle: NSWarningAlertStyle];
-        
-        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
-        return;
-    }
-    
-    //check if a file with the same name and location already exists
-    if ([[NSFileManager defaultManager] fileExistsAtPath: fLocation])
-    {
-        NSArray * pathComponents = [fLocation pathComponents];
-        int count = [pathComponents count];
-        
-        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
-        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> file already exists warning -> button")];
-        [alert setMessageText: NSLocalizedString(@"A torrent file with this name and directory cannot be created.",
-                                                "Create torrent -> file already exists warning -> title")];
-        [alert setInformativeText: [NSString stringWithFormat:
-                NSLocalizedString(@"A file with the name \"%@\" already exists in the directory \"%@\". "
-                    "Choose a new name or directory to create the torrent.",
-                    "Create torrent -> file already exists warning -> warning"),
-                    [pathComponents objectAtIndex: count-1], [pathComponents objectAtIndex: count-2]]];
-        [alert setAlertStyle: NSWarningAlertStyle];
-        
-        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
-        return;
-    }
-    
-    fOpenTorrent = [fOpenCheck state] == NSOnState;
-    
-    //store values
-    [fDefaults setObject: trackerString forKey: @"CreatorTracker"];
-    [fDefaults setBool: [fPrivateCheck state] == NSOnState forKey: @"CreatorPrivate"];
-    [fDefaults setBool: fOpenTorrent forKey: @"CreatorOpen"];
-    [fDefaults setObject: [fLocation stringByDeletingLastPathComponent] forKey: @"CreatorLocation"];
-    
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"BeginCreateTorrentFile" object: fLocation userInfo: nil];
-    tr_makeMetaInfo(fInfo, [fLocation UTF8String], [trackerString UTF8String], [[fCommentView string] UTF8String],
-                    [fPrivateCheck state] == NSOnState);
-    
-    fTimer = [NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(checkProgress)
-                        userInfo: nil repeats: YES];
+        [self createReal];
 }
 
 - (void) cancelCreateWindow: (id) sender
@@ -277,6 +269,72 @@
     [fTimer fire];
 }
 
+- (NSInteger) numberOfRowsInTableView: (NSTableView *) tableView
+{
+    return [fTrackers count];
+}
+
+- (id) tableView: (NSTableView *) tableView objectValueForTableColumn: (NSTableColumn *) tableColumn row: (NSInteger) row
+{
+    return [fTrackers objectAtIndex: row];
+}
+
+- (void) addRemoveTracker: (id) sender
+{
+    //don't allow add/remove when currently adding - it leads to weird results
+    if ([fTrackerTable editedRow] != -1)
+        return;
+    
+    if ([[sender cell] tagForSegment: [sender selectedSegment]] == TRACKER_REMOVE_TAG)
+    {
+        [fTrackers removeObjectsAtIndexes: [fTrackerTable selectedRowIndexes]];
+        
+        [fTrackerTable deselectAll: self];
+        [fTrackerTable reloadData];
+        
+        [self updateEnableOpenCheckForTrackers];
+    }
+    else
+    {
+        [fTrackers addObject: @""];
+        [fTrackerTable reloadData];
+        
+        NSInteger row = [fTrackers count] - 1;
+        [fTrackerTable selectRow: row byExtendingSelection: NO];
+        [fTrackerTable editColumn: 0 row: row withEvent: nil select: YES];
+    }
+}
+
+- (void) tableView: (NSTableView *) tableView setObjectValue: (id) object forTableColumn: (NSTableColumn *) tableColumn
+    row: (NSInteger) row
+{
+    NSString * tracker = (NSString *)object;
+    
+    tracker = [tracker stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+    
+    if ([tracker rangeOfString: @"://"].location == NSNotFound)
+        tracker = [@"http://" stringByAppendingString: tracker];
+    
+    if (!tr_httpIsValidURL([tracker UTF8String]))
+    {
+        NSBeep();
+        [fTrackers removeObjectAtIndex: row];
+    }
+    else
+    {
+        [fTrackers replaceObjectAtIndex: row withObject: tracker];
+        [self updateEnableOpenCheckForTrackers];
+    }
+    
+    [fTrackerTable deselectAll: self];
+    [fTrackerTable reloadData];
+}
+
+- (void) tableViewSelectionDidChange: (NSNotification *) notification
+{
+    [fTrackerAddRemoveControl setEnabled: [fTrackerTable numberOfSelectedRows] > 0 forSegment: TRACKER_REMOVE_TAG];
+}
+
 @end
 
 @implementation CreatorWindowController (Private)
@@ -298,7 +356,14 @@
     return success ? [[panel filenames] objectAtIndex: 0] : nil;
 }
 
-- (void) locationSheetClosed: (NSSavePanel *) panel returnCode: (int) code contextInfo: (void *) info
+- (void) updateEnableOpenCheckForTrackers
+{
+    BOOL hasTracker = [fTrackers count] > 0;
+    [fOpenCheck setEnabled: hasTracker];
+    [fOpenCheck setState: (fOpenTorrent && hasTracker) ? NSOnState : NSOffState];
+}
+
+- (void) locationSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (void *) info
 {
     if (code == NSOKButton)
     {
@@ -310,6 +375,61 @@
     }
 }
 
+- (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo
+{
+    if (([NSApp isOnLeopardOrBetter] ? [[alert suppressionButton] state] == NSOnState : returnCode == NSAlertThirdButtonReturn))
+        [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorBlankAddress"];
+    
+    [alert release];
+    
+    if (returnCode == NSAlertFirstButtonReturn)
+        [self performSelectorOnMainThread: @selector(createReal) withObject: nil waitUntilDone: NO];
+}
+
+- (void) createReal
+{
+    //check if a file with the same name and location already exists
+    if ([[NSFileManager defaultManager] fileExistsAtPath: fLocation])
+    {
+        NSArray * pathComponents = [fLocation pathComponents];
+        NSInteger count = [pathComponents count];
+        
+        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
+        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> file already exists warning -> button")];
+        [alert setMessageText: NSLocalizedString(@"A torrent file with this name and directory cannot be created.",
+                                                "Create torrent -> file already exists warning -> title")];
+        [alert setInformativeText: [NSString stringWithFormat:
+                NSLocalizedString(@"A file with the name \"%@\" already exists in the directory \"%@\". "
+                    "Choose a new name or directory to create the torrent file.",
+                    "Create torrent -> file already exists warning -> warning"),
+                    [pathComponents objectAtIndex: count-1], [pathComponents objectAtIndex: count-2]]];
+        [alert setAlertStyle: NSWarningAlertStyle];
+        
+        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
+        return;
+    }
+    
+    //parse non-empty tracker strings
+    tr_tracker_info * trackerInfo = tr_new0(tr_tracker_info, [fTrackers count]);
+    
+    for (NSUInteger i = 0; i < [fTrackers count]; i++)
+        trackerInfo[i].announce = (char *)[[fTrackers objectAtIndex: i] UTF8String];
+    
+    //store values
+    [fDefaults setObject: fTrackers forKey: @"CreatorTrackers"];
+    [fDefaults setBool: [fPrivateCheck state] == NSOnState forKey: @"CreatorPrivate"];
+    [fDefaults setBool: fOpenTorrent forKey: @"CreatorOpen"];
+    [fDefaults setObject: [fLocation stringByDeletingLastPathComponent] forKey: @"CreatorLocation"];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"BeginCreateTorrentFile" object: fLocation userInfo: nil];
+    tr_makeMetaInfo(fInfo, [fLocation UTF8String], trackerInfo, [fTrackers count], [[fCommentView string] UTF8String],
+                    [fPrivateCheck state] == NSOnState);
+    tr_free(trackerInfo);
+    
+    fTimer = [NSTimer scheduledTimerWithTimeInterval: 0.1 target: self selector: @selector(checkProgress)
+                userInfo: nil repeats: YES];
+}
+
 - (void) checkProgress
 {
     if (fInfo->isDone)
@@ -317,34 +437,42 @@
         [fTimer invalidate];
         fTimer = nil;
         
-        if (fInfo->failed)
+        NSAlert * alert;
+        switch (fInfo->result)
         {
-            if (!fInfo->abortFlag)
-            {
-                NSAlert * alert = [[[NSAlert alloc] init] autorelease];
+            case TR_MAKEMETA_OK:
+                if (fOpenTorrent && [fTrackers count] > 0)
+                {
+                    NSDictionary * dict = [[NSDictionary alloc] initWithObjectsAndKeys: fLocation, @"File",
+                                            [fPath stringByDeletingLastPathComponent], @"Path", nil];
+                    [[NSNotificationCenter defaultCenter] postNotificationName: @"OpenCreatedTorrentFile" object: self userInfo: dict];
+                }
+                
+                [[self window] close];
+                break;
+            
+            case TR_MAKEMETA_CANCELLED:
+                [[self window] close];
+                break;
+            
+            default:
+                alert = [[[NSAlert alloc] init] autorelease];
                 [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> failed -> button")];
                 [alert setMessageText: [NSString stringWithFormat: NSLocalizedString(@"Creation of \"%@\" failed.",
                                                 "Create torrent -> failed -> title"), [fLocation lastPathComponent]]];
-                [alert setInformativeText: NSLocalizedString(@"There was an error parsing the data file. "
-                                            "The torrent file was not created.", "Create torrent -> failed -> warning")];
                 [alert setAlertStyle: NSWarningAlertStyle];
                 
+                if (fInfo->result == TR_MAKEMETA_IO_READ)
+                    [alert setInformativeText: [NSString stringWithFormat: NSLocalizedString(@"Could not read \"%s\": %s.",
+                        "Create torrent -> failed -> warning"), fInfo->errfile, strerror(fInfo->my_errno)]];
+                else if (fInfo->result == TR_MAKEMETA_IO_WRITE)
+                    [alert setInformativeText: [NSString stringWithFormat: NSLocalizedString(@"Could not write \"%s\": %s.",
+                        "Create torrent -> failed -> warning"), fInfo->errfile, strerror(fInfo->my_errno)]];
+                else; //invalid url should have been caught before creating
+                
                 [alert beginSheetModalForWindow: [self window] modalDelegate: self
                         didEndSelector: @selector(failureSheetClosed:returnCode:contextInfo:) contextInfo: nil];
-                return;
-            }
         }
-        else
-        {
-            if (fOpenTorrent)
-            {
-                NSDictionary * dict = [[NSDictionary alloc] initWithObjectsAndKeys: fLocation, @"File",
-                                        [fPath stringByDeletingLastPathComponent], @"Path", nil];
-                [[NSNotificationCenter defaultCenter] postNotificationName: @"OpenCreatedTorrentFile" object: self userInfo: dict];
-            }
-        }
-        
-        [[self window] close];
     }
     else
     {
@@ -352,15 +480,19 @@
         
         if (!fStarted)
         {
+            fStarted = YES;
+            
+            [fProgressView setHidden: YES];
+            
             NSWindow * window = [self window];
             
             NSRect windowRect = [window frame];
-            float difference = [fProgressView frame].size.height - [[window contentView] frame].size.height;
+            CGFloat difference = [fProgressView frame].size.height - [[window contentView] frame].size.height;
             windowRect.origin.y -= difference;
             windowRect.size.height += difference;
             
             //don't allow vertical resizing
-            float height = windowRect.size.height;
+            CGFloat height = windowRect.size.height;
             [window setMinSize: NSMakeSize([window minSize].width, height)];
             [window setMaxSize: NSMakeSize([window maxSize].width, height)];
             
@@ -368,14 +500,12 @@
             [window setFrame: windowRect display: YES animate: YES];
             [fProgressView setHidden: NO];
             
-            [[window standardWindowButton:NSWindowCloseButton] setEnabled: NO];
-            
-            fStarted = YES;
+            [[window standardWindowButton: NSWindowCloseButton] setEnabled: NO];
         }
     }
 }
 
-- (void) failureSheetClosed: (NSAlert *) alert returnCode: (int) code contextInfo: (void *) info
+- (void) failureSheetClosed: (NSAlert *) alert returnCode: (NSInteger) code contextInfo: (void *) info
 {
     [[alert window] orderOut: nil];
     [[self window] close];
