@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: TorrentCell.m 5460 2008-03-31 18:39:44Z livings124 $
+ * $Id: TorrentCell.m 7032 2008-11-04 01:31:24Z livings124 $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -24,37 +24,39 @@
 
 #import "TorrentCell.h"
 #import "TorrentTableView.h"
-#import "GroupsWindowController.h"
+#import "GroupsController.h"
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "NSBezierPathAdditions.h"
-#import "CTGradientAdditions.h"
+#import "ProgressGradients.h"
+#import "CTGradient.h"
 
-#define BAR_HEIGHT 12.0
+#define BAR_HEIGHT 12.0f
 
-#define IMAGE_SIZE_REG 32.0
-#define IMAGE_SIZE_MIN 16.0
+#define IMAGE_SIZE_REG 32.0f
+#define IMAGE_SIZE_MIN 16.0f
 
-#define NORMAL_BUTTON_WIDTH 14.0
-#define ACTION_BUTTON_WIDTH 16.0
+#define NORMAL_BUTTON_WIDTH 14.0f
+#define ACTION_BUTTON_WIDTH 16.0f
 
 //ends up being larger than font height
-#define HEIGHT_TITLE 16.0
-#define HEIGHT_STATUS 12.0
+#define HEIGHT_TITLE 16.0f
+#define HEIGHT_STATUS 12.0f
 
-#define PADDING_HORIZONTAL 3.0
-#define PADDING_BETWEEN_IMAGE_AND_TITLE 5.0
-#define PADDING_BETWEEN_IMAGE_AND_BAR 7.0
-#define PADDING_ABOVE_TITLE 4.0
-#define PADDING_ABOVE_MIN_STATUS 4.0
-#define PADDING_BETWEEN_TITLE_AND_MIN_STATUS 2.0
-#define PADDING_BETWEEN_TITLE_AND_PROGRESS 1.0
-#define PADDING_BETWEEN_PROGRESS_AND_BAR 2.0
-#define PADDING_BETWEEN_TITLE_AND_BAR_MIN 3.0
-#define PADDING_BETWEEN_BAR_AND_STATUS 2.0
+#define PADDING_HORIZONTAL 3.0f
+#define PADDING_BETWEEN_IMAGE_AND_TITLE 5.0f
+#define PADDING_BETWEEN_IMAGE_AND_BAR 7.0f
+#define PADDING_ABOVE_TITLE 4.0f
+#define PADDING_ABOVE_MIN_STATUS 4.0f
+#define PADDING_BETWEEN_TITLE_AND_MIN_STATUS 2.0f
+#define PADDING_BETWEEN_TITLE_AND_PROGRESS 1.0f
+#define PADDING_BETWEEN_PROGRESS_AND_BAR 2.0f
+#define PADDING_BETWEEN_TITLE_AND_BAR_MIN 3.0f
+#define PADDING_BETWEEN_BAR_AND_STATUS 2.0f
 
-#define PIECES_TOTAL_PERCENT 0.6
-#define MAX_PIECES 324
+#define PIECES_TOTAL_PERCENT 0.6f
+
+#define MAX_PIECES (18*18)
 
 @interface TorrentCell (Private)
 
@@ -90,74 +92,26 @@
         [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
     
         fTitleAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                                [NSFont messageFontOfSize: 12.0], NSFontAttributeName,
+                                [NSFont messageFontOfSize: 12.0f], NSFontAttributeName,
                                 paragraphStyle, NSParagraphStyleAttributeName, nil];
         fStatusAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                                [NSFont messageFontOfSize: 9.0], NSFontAttributeName,
+                                [NSFont messageFontOfSize: 9.0f], NSFontAttributeName,
                                 paragraphStyle, NSParagraphStyleAttributeName, nil];
         [paragraphStyle release];
         
-        //store box colors
-        fGrayColor = [[NSColor colorWithCalibratedRed: 0.9 green: 0.9 blue: 0.9 alpha: 1.0] retain];
-        fBlue1Color = [[NSColor colorWithCalibratedRed: 0.8 green: 1.0 blue: 1.0 alpha: 1.0] retain];
-        fBlue2Color = [[NSColor colorWithCalibratedRed: 0.6 green: 1.0 blue: 1.0 alpha: 1.0] retain];
-        fBlue3Color = [[NSColor colorWithCalibratedRed: 0.6 green: 0.8 blue: 1.0 alpha: 1.0] retain];
-        fBlue4Color = [[NSColor colorWithCalibratedRed: 0.4 green: 0.6 blue: 1.0 alpha: 1.0] retain];
-        fBlueColor = [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.8 alpha: 1.0] retain];
-        fOrangeColor = [[NSColor orangeColor] retain];
-        
-        fBarOverlayColor = [[NSColor colorWithDeviceWhite: 0.0 alpha: 0.2] retain];
-        
-        fFieldBackColor = [[NSColor colorWithDeviceWhite: 0.65 alpha: 0.7] retain];
+        fBluePieceColor = [[NSColor colorWithCalibratedRed: 0.0f green: 0.4f blue: 0.8f alpha: 1.0f] retain];
+        fBarBorderColor = [[NSColor colorWithDeviceWhite: 0.0f alpha: 0.2f] retain];
     }
 	return self;
 }
 
-- (id) copyWithZone: (NSZone *) zone
-{
-    TorrentCell * copy = [super copyWithZone: zone];
-    
-    copy->fBitmap = nil;
-    
-    copy->fGrayGradient = [fGrayGradient retain];
-    copy->fLightGrayGradient = [fLightGrayGradient retain];
-    copy->fBlueGradient = [fBlueGradient retain];
-    copy->fDarkBlueGradient = [fDarkBlueGradient retain];
-    copy->fGreenGradient = [fGreenGradient retain];
-    copy->fLightGreenGradient = [fLightGreenGradient retain];
-    copy->fDarkGreenGradient = [fDarkGreenGradient retain];
-    copy->fYellowGradient = [fYellowGradient retain];
-    copy->fRedGradient = [fRedGradient retain];
-    copy->fTransparentGradient = [fTransparentGradient retain];
-    
-    return copy;
-}
-
-- (void) dealloc
-{
-    [fBitmap release];
-    
-    [fGrayGradient release];
-    [fLightGrayGradient release];
-    [fBlueGradient release];
-    [fDarkBlueGradient release];
-    [fGreenGradient release];
-    [fLightGreenGradient release];
-    [fDarkGreenGradient release];
-    [fYellowGradient release];
-    [fRedGradient release];
-    [fTransparentGradient release];
-    
-    [super dealloc];
-}
-
 - (NSRect) iconRectForBounds: (NSRect) bounds
 {
-    float imageSize = [fDefaults boolForKey: @"SmallView"] ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG;
+    CGFloat imageSize = [fDefaults boolForKey: @"SmallView"] ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG;
     
     NSRect result = bounds;
     result.origin.x += PADDING_HORIZONTAL;
-    result.origin.y += floorf((result.size.height - imageSize) * 0.5);
+    result.origin.y += floorf((result.size.height - imageSize) * 0.5f);
     result.size = NSMakeSize(imageSize, imageSize);
     
     return result;
@@ -171,12 +125,18 @@
 
 - (NSRect) minimalStatusRectForBounds: (NSRect) bounds
 {
+    if (![fDefaults boolForKey: @"SmallView"])
+        return NSZeroRect;
+    
     return [self rectForMinimalStatusWithString: [self attributedStatusString: [self minimalStatusString] withColor: nil]
             inBounds: bounds];
 }
 
 - (NSRect) progressRectForBounds: (NSRect) bounds
 {
+    if ([fDefaults boolForKey: @"SmallView"])
+        return NSZeroRect;
+    
     return [self rectForProgressWithString: [self attributedStatusString: [[self representedObject] progressString] withColor: nil]
             inBounds: bounds];
 }
@@ -195,13 +155,16 @@
     else
         result.origin.y += PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS + PADDING_BETWEEN_PROGRESS_AND_BAR;
     
-    result.size.width = round(NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL - 2.0 * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH));
+    result.size.width = round(NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL - 2.0f * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH));
     
     return result;
 }
 
 - (NSRect) statusRectForBounds: (NSRect) bounds
 {
+    if ([fDefaults boolForKey: @"SmallView"])
+        return NSZeroRect;
+    
     return [self rectForStatusWithString: [self attributedStatusString: [self statusString] withColor: nil] inBounds: bounds];
 }
 
@@ -210,9 +173,9 @@
     NSRect result = bounds;
     result.size.height = NORMAL_BUTTON_WIDTH;
     result.size.width = NORMAL_BUTTON_WIDTH;
-    result.origin.x = NSMaxX(bounds) - 2.0 * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH);
+    result.origin.x = NSMaxX(bounds) - 2.0f * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH);
     
-    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5;
+    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5f;
     if ([fDefaults boolForKey: @"SmallView"])
         result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
     else
@@ -228,7 +191,7 @@
     result.size.width = NORMAL_BUTTON_WIDTH;
     result.origin.x = NSMaxX(bounds) - (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH);
     
-    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5;
+    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5f;
     if ([fDefaults boolForKey: @"SmallView"])
         result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
     else
@@ -239,15 +202,14 @@
 
 - (NSRect) actionButtonRectForBounds: (NSRect) bounds
 {
-    NSRect iconRect = [self iconRectForBounds: bounds];
-    if ([fDefaults boolForKey: @"SmallView"])
-        return iconRect;
-    
-    NSRect result = iconRect;
-    result.origin.x += (iconRect.size.width - ACTION_BUTTON_WIDTH) * 0.5;
-    result.origin.y += (iconRect.size.height - ACTION_BUTTON_WIDTH) * 0.5;
-    result.size.width = ACTION_BUTTON_WIDTH;
-    result.size.height = ACTION_BUTTON_WIDTH;
+    NSRect result = [self iconRectForBounds: bounds];
+    if (![fDefaults boolForKey: @"SmallView"])
+    {
+        result.origin.x += (result.size.width - ACTION_BUTTON_WIDTH) * 0.5f;
+        result.origin.y += (result.size.height - ACTION_BUTTON_WIDTH) * 0.5f;
+        result.size.width = ACTION_BUTTON_WIDTH;
+        result.size.height = ACTION_BUTTON_WIDTH;
+    }
     
     return result;
 }
@@ -257,9 +219,7 @@
     NSPoint point = [controlView convertPoint: [event locationInWindow] fromView: nil];
     
     if (NSMouseInRect(point, [self controlButtonRectForBounds: cellFrame], [controlView isFlipped])
-        || NSMouseInRect(point, [self revealButtonRectForBounds: cellFrame], [controlView isFlipped])
-        || (NSMouseInRect(point, [self progressRectForBounds: cellFrame], [controlView isFlipped]) && [[self representedObject] folder])
-        || NSMouseInRect(point, [self minimalStatusRectForBounds: cellFrame], [controlView isFlipped]))
+        || NSMouseInRect(point, [self revealButtonRectForBounds: cellFrame], [controlView isFlipped]))
         return NSCellHitContentArea | NSCellHitTrackableArea;
     
     return NSCellHitContentArea;
@@ -284,12 +244,6 @@
     NSRect revealRect = [self revealButtonRectForBounds: cellFrame];
     BOOL checkReveal = NSMouseInRect(point, revealRect, [controlView isFlipped]);
     
-    NSRect progressRect = [self progressRectForBounds: cellFrame];
-    BOOL checkProgress = NSMouseInRect(point, progressRect, [controlView isFlipped]) && [[self representedObject] folder];
-    
-    NSRect minimalStatusRect = [self minimalStatusRectForBounds: cellFrame];
-    BOOL checkMinStatus = NSMouseInRect(point, minimalStatusRect, [controlView isFlipped]);
-    
     [(TorrentTableView *)controlView removeButtonTrackingAreas];
     
     while ([event type] != NSLeftMouseUp)
@@ -314,24 +268,6 @@
                 [controlView setNeedsDisplayInRect: cellFrame];
             }
         }
-        else if (checkProgress)
-        {
-            BOOL inProgressField = NSMouseInRect(point, progressRect, [controlView isFlipped]);
-            if (fMouseDownProgressField != inProgressField)
-            {
-                fMouseDownProgressField = inProgressField;
-                [controlView setNeedsDisplayInRect: cellFrame];
-            }
-        }
-        else if (checkMinStatus)
-        {
-            BOOL inMinStatusField = NSMouseInRect(point, minimalStatusRect, [controlView isFlipped]);
-            if (fMouseDownMinimalStatusField != inMinStatusField)
-            {
-                fMouseDownMinimalStatusField = inMinStatusField;
-                [controlView setNeedsDisplayInRect: cellFrame];
-            }
-        }
         else;
         
         //send events to where necessary
@@ -356,20 +292,6 @@
         
         [[self representedObject] revealData];
     }
-    else if (fMouseDownProgressField)
-    {
-        fMouseDownProgressField = NO;
-        
-        [fDefaults setBool: ![fDefaults boolForKey: @"DisplayStatusProgressSelected"] forKey: @"DisplayStatusProgressSelected"];
-        [(TorrentTableView *)controlView reloadData];
-    }
-    else if (fMouseDownMinimalStatusField)
-    {
-        fMouseDownMinimalStatusField = NO;
-        
-        [fDefaults setBool: ![fDefaults boolForKey: @"DisplaySmallStatusRegular"] forKey: @"DisplaySmallStatusRegular"];
-        [(TorrentTableView *)controlView reloadData];
-    }
     else;
     
     if ([NSApp isOnLeopardOrBetter])
@@ -435,17 +357,17 @@
 
 - (void) setControlHover: (BOOL) hover
 {
-    fHoverControl = [NSApp isOnLeopardOrBetter] ? hover : NO;
+    fHoverControl = hover;
 }
 
 - (void) setRevealHover: (BOOL) hover
 {
-    fHoverReveal = [NSApp isOnLeopardOrBetter] ? hover : NO;
+    fHoverReveal = hover;
 }
 
 - (void) setActionHover: (BOOL) hover
 {
-    fHoverAction = [NSApp isOnLeopardOrBetter] ? hover : NO;
+    fHoverAction = hover;
 }
 
 - (void) setActionPushed: (BOOL) pushed
@@ -462,18 +384,31 @@
     //group coloring
     NSRect iconRect = [self iconRectForBounds: cellFrame];
     
-    int groupValue = [torrent groupValue];
+    NSInteger groupValue = [torrent groupValue];
     if (groupValue != -1)
     {
-        NSRect groupRect = NSInsetRect(iconRect, -2.0, -3.0);
+        NSRect groupRect = NSInsetRect(iconRect, -1.0f, -2.0f);
         if (!minimal)
         {
             groupRect.size.height--;
             groupRect.origin.y--;
         }
+        const CGFloat radius = minimal ? 3.0f : 6.0f;
         
-        [[[GroupsWindowController groups] gradientForIndex: groupValue] fillBezierPath:
-            [NSBezierPath bezierPathWithRoundedRect: groupRect radius: 6.0] angle: 90.0];
+        NSColor * groupColor = [[GroupsController groups] colorForIndex: groupValue],
+                * darkGroupColor = [groupColor blendedColorWithFraction: 0.2f ofColor: [NSColor whiteColor]];
+        
+        //border
+        NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: groupRect radius: radius];
+        [darkGroupColor set];
+        [bp setLineWidth: 2.0f];
+        [bp stroke];
+        
+        //inside
+        bp = [NSBezierPath bezierPathWithRoundedRect: groupRect radius: radius];
+        CTGradient * gradient = [CTGradient gradientWithBeginningColor: [groupColor blendedColorWithFraction: 0.7f
+                                    ofColor: [NSColor whiteColor]] endingColor: darkGroupColor];
+        [gradient fillBezierPath: bp angle: 90.0f];
     }
     
     //error image
@@ -488,19 +423,19 @@
     if (!minimal || !(!fTracking && fHoverAction)) //don't show in minimal mode when hovered over
     {
         NSImage * icon = (minimal && error) ? fErrorImage : [torrent icon];
-        [icon drawInRect: iconRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+        [icon drawInRect: iconRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0f];
     }
     
     if (error && !minimal)
     {
         NSRect errorRect = NSMakeRect(NSMaxX(iconRect) - IMAGE_SIZE_MIN, NSMaxY(iconRect) - IMAGE_SIZE_MIN,
                                         IMAGE_SIZE_MIN, IMAGE_SIZE_MIN);
-        [fErrorImage drawInRect: errorRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+        [fErrorImage drawInRect: errorRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0f];
     }
     
     //text color
     NSColor * titleColor, * statusColor;
-    if ([NSApp isOnLeopardOrBetter] ? [self backgroundStyle] == NSBackgroundStyleDark : [self isHighlighted]
+    if ([self isHighlighted]
         && [[self highlightColorWithFrame: cellFrame inView: controlView] isEqual: [NSColor alternateSelectedControlColor]])
     {
         titleColor = [NSColor whiteColor];
@@ -519,12 +454,6 @@
         NSAttributedString * minimalString = [self attributedStatusString: [self minimalStatusString] withColor: statusColor];
         minimalStatusRect = [self rectForMinimalStatusWithString: minimalString inBounds: cellFrame];
         
-        if (fMouseDownMinimalStatusField)
-        {
-            [fFieldBackColor set];
-            [[NSBezierPath bezierPathWithRoundedRect: NSInsetRect(minimalStatusRect, -2.0, 0.0) radius: 5.0] fill];
-        }
-        
         [minimalString drawInRect: minimalStatusRect];
     }
     
@@ -539,12 +468,6 @@
         NSAttributedString * progressString = [self attributedStatusString: [torrent progressString] withColor: statusColor];
         NSRect progressRect = [self rectForProgressWithString: progressString inBounds: cellFrame];
         
-        if (fMouseDownProgressField)
-        {
-            [fFieldBackColor set];
-            [[NSBezierPath bezierPathWithRoundedRect: NSInsetRect(progressRect, -2.0, 0.0) radius: 5.0] fill];
-        }
-        
         [progressString drawInRect: progressRect];
     }
     
@@ -575,37 +498,37 @@
     
     [controlImage setFlipped: YES];
     [controlImage drawInRect: [self controlButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
-        fraction: 1.0];
+        fraction: 1.0f];
     
     //reveal button
-    NSString * revealImageSuffix;
+    NSString * revealImageString;
     if (fMouseDownRevealButton)
-        revealImageSuffix = @"On.png";
+        revealImageString = @"RevealOn.png";
     else if (!fTracking && fHoverReveal)
-        revealImageSuffix = @"Hover.png";
+        revealImageString = @"RevealHover.png";
     else
-        revealImageSuffix = @"Off.png";
+        revealImageString = @"RevealOff.png";
     
-    NSImage * revealImage = [NSImage imageNamed: [@"Reveal" stringByAppendingString: revealImageSuffix]];
+    NSImage * revealImage = [NSImage imageNamed: revealImageString];
     [revealImage setFlipped: YES];
     [revealImage drawInRect: [self revealButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
-        fraction: 1.0];
+        fraction: 1.0f];
     
     //action button
-    NSString * actionImageSuffix;
+    NSString * actionImageString;
     if (fMouseDownActionButton)
-        actionImageSuffix = @"On.png";
+        actionImageString = @"ActionOn.png";
     else if (!fTracking && fHoverAction)
-        actionImageSuffix = @"Hover.png";
+        actionImageString = @"ActionHover.png";
     else
-        actionImageSuffix = nil;
+        actionImageString = nil;
     
-    if (actionImageSuffix)
+    if (actionImageString)
     {
-        NSImage * actionImage = [NSImage imageNamed: [@"Action" stringByAppendingString: actionImageSuffix]];
+        NSImage * actionImage = [NSImage imageNamed: actionImageString];
         [actionImage setFlipped: YES];
         [actionImage drawInRect: [self actionButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
-            fraction: 1.0];
+            fraction: 1.0f];
     }
     
     //status
@@ -622,8 +545,8 @@
 
 - (void) drawBar: (NSRect) barRect
 {
-    float piecesBarPercent = [(TorrentTableView *)[self controlView] piecesBarPercent];
-    if (piecesBarPercent > 0.0)
+    CGFloat piecesBarPercent = [(TorrentTableView *)[self controlView] piecesBarPercent];
+    if (piecesBarPercent > 0.0f)
     {
         NSRect regularBarRect = barRect, piecesBarRect = barRect;
         piecesBarRect.size.height *= PIECES_TOTAL_PERCENT * piecesBarPercent;
@@ -635,42 +558,38 @@
     }
     else
     {
-        [fBitmap release];
-        fBitmap = nil;
-        [[self representedObject] setPreviousAmountFinished: NULL];
+        [[self representedObject] setPreviousFinishedPieces: nil];
         
         [self drawRegularBar: barRect];
     }
     
-    [fBarOverlayColor set];
-    [NSBezierPath strokeRect: NSInsetRect(barRect, 0.5, 0.5)];
+    [fBarBorderColor set];
+    [NSBezierPath strokeRect: NSInsetRect(barRect, 0.5f, 0.5f)];
 }
 
 - (void) drawRegularBar: (NSRect) barRect
 {
     Torrent * torrent = [self representedObject];
     
-    int leftWidth = barRect.size.width;
-    float progress = [torrent progress];
+    NSInteger leftWidth = barRect.size.width;
+    CGFloat progress = [torrent progress];
     
-    if (progress < 1.0)
+    if (progress < 1.0f)
     {
-        float rightProgress = 1.0 - progress, progressLeft = [torrent progressLeft];
-        int rightWidth = leftWidth * rightProgress;
+        CGFloat rightProgress = 1.0f - progress, progressLeft = [torrent progressLeft];
+        NSInteger rightWidth = leftWidth * rightProgress;
         leftWidth -= rightWidth;
         
         if (progressLeft < rightProgress)
         {
-            int rightNoIncludeWidth = rightWidth * ((rightProgress - progressLeft) / rightProgress);
+            NSInteger rightNoIncludeWidth = rightWidth * ((rightProgress - progressLeft) / rightProgress);
             rightWidth -= rightNoIncludeWidth;
             
             NSRect noIncludeRect = barRect;
             noIncludeRect.origin.x += barRect.size.width - rightNoIncludeWidth;
             noIncludeRect.size.width = rightNoIncludeWidth;
             
-            if (!fLightGrayGradient)
-                fLightGrayGradient = [[CTGradient progressLightGrayGradient] retain];
-            [fLightGrayGradient fillRect: noIncludeRect angle: -90];
+            [[ProgressGradients progressLightGrayGradient] fillRect: noIncludeRect angle: 90];
         }
         
         if (rightWidth > 0)
@@ -678,18 +597,16 @@
             if ([torrent isActive] && ![torrent allDownloaded] && ![torrent isChecking]
                 && [fDefaults boolForKey: @"DisplayProgressBarAvailable"])
             {
-                int notAvailableWidth = MIN(ceil(barRect.size.width * [torrent notAvailableDesired]), rightWidth);
-                rightWidth -= notAvailableWidth;
-                
+                NSInteger notAvailableWidth = ceil(rightWidth * [torrent notAvailableDesired]);
                 if (notAvailableWidth > 0)
                 {
+                    rightWidth -= notAvailableWidth;
+                    
                     NSRect notAvailableRect = barRect;
                     notAvailableRect.origin.x += leftWidth + rightWidth;
                     notAvailableRect.size.width = notAvailableWidth;
                     
-                    if (!fRedGradient)
-                        fRedGradient = [[CTGradient progressRedGradient] retain];
-                    [fRedGradient fillRect: notAvailableRect angle: -90];
+                    [[ProgressGradients progressRedGradient] fillRect: notAvailableRect angle: 90];
                 }
             }
             
@@ -699,9 +616,7 @@
                 includeRect.origin.x += leftWidth;
                 includeRect.size.width = rightWidth;
                 
-                if (!fWhiteGradient)
-                    fWhiteGradient = [[CTGradient progressWhiteGradient] retain];
-                [fWhiteGradient fillRect: includeRect angle: -90];
+                [[ProgressGradients progressWhiteGradient] fillRect: includeRect angle: 90];
             }
         }
     }
@@ -714,14 +629,10 @@
         if ([torrent isActive])
         {
             if ([torrent isChecking])
-            {
-                if (!fYellowGradient)
-                    fYellowGradient = [[CTGradient progressYellowGradient] retain];
-                [fYellowGradient fillRect: completeRect angle: -90];
-            }
+                [[ProgressGradients progressYellowGradient] fillRect: completeRect angle: 90];
             else if ([torrent isSeeding])
             {
-                int ratioLeftWidth = leftWidth * (1.0 - [torrent progressStopRatio]);
+                NSInteger ratioLeftWidth = leftWidth * (1.0f - [torrent progressStopRatio]);
                 leftWidth -= ratioLeftWidth;
                 
                 if (ratioLeftWidth > 0)
@@ -730,111 +641,78 @@
                     ratioLeftRect.origin.x += leftWidth;
                     ratioLeftRect.size.width = ratioLeftWidth;
                     
-                    if (!fLightGreenGradient)
-                        fLightGreenGradient = [[CTGradient progressLightGreenGradient] retain];
-                    [fLightGreenGradient fillRect: ratioLeftRect angle: -90];
+                    [[ProgressGradients progressLightGreenGradient] fillRect: ratioLeftRect angle: 90];
                 }
                 
                 if (leftWidth > 0)
                 {
                     completeRect.size.width = leftWidth;
                     
-                    if (!fGreenGradient)
-                        fGreenGradient = [[CTGradient progressGreenGradient] retain];
-                    [fGreenGradient fillRect: completeRect angle: -90];
+                    [[ProgressGradients progressGreenGradient] fillRect: completeRect angle: 90];
                 }
             }
             else
-            {
-                if (!fBlueGradient)
-                    fBlueGradient = [[CTGradient progressBlueGradient] retain];
-                [fBlueGradient fillRect: completeRect angle: -90];
-            }
+                [[ProgressGradients progressBlueGradient] fillRect: completeRect angle: 90];
         }
         else
         {
             if ([torrent waitingToStart])
             {
-                if ([torrent progressLeft] <= 0.0)
-                {
-                    if (!fDarkGreenGradient)
-                        fDarkGreenGradient = [[CTGradient progressDarkGreenGradient] retain];
-                    [fDarkGreenGradient fillRect: completeRect angle: -90];
-                }
+                if ([torrent progressLeft] <= 0.0f)
+                    [[ProgressGradients progressDarkGreenGradient] fillRect: completeRect angle: 90];
                 else
-                {
-                    if (!fDarkBlueGradient)
-                        fDarkBlueGradient = [[CTGradient progressDarkBlueGradient] retain];
-                    [fDarkBlueGradient fillRect: completeRect angle: -90];
-                }
+                    [[ProgressGradients progressDarkBlueGradient] fillRect: completeRect angle: 90];
             }
             else
-            {
-                if (!fGrayGradient)
-                    fGrayGradient = [[CTGradient progressGrayGradient] retain];
-                [fGrayGradient fillRect: completeRect angle: -90];
-            }
+                [[ProgressGradients progressGrayGradient] fillRect: completeRect angle: 90];
         }
     }
 }
 
 - (void) drawPiecesBar: (NSRect) barRect
 {
-    if (!fBitmap)
-        fBitmap = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes: nil
-            pixelsWide: MAX_PIECES pixelsHigh: 1 bitsPerSample: 8 samplesPerPixel: 4 hasAlpha: YES
-            isPlanar: NO colorSpaceName: NSCalibratedRGBColorSpace bytesPerRow: 0 bitsPerPixel: 0];
-    
     Torrent * torrent = [self representedObject];
     
-    int pieceCount = MIN([torrent pieceCount], MAX_PIECES);
-    float * piecePercent = malloc(pieceCount * sizeof(float)),
-        * previousPiecePercent = [torrent getPreviousAmountFinished];
-    [torrent getAmountFinished: piecePercent size: pieceCount];
+    NSInteger pieceCount = MIN([torrent pieceCount], MAX_PIECES);
+    float * piecesPercent = malloc(pieceCount * sizeof(float));
+    [torrent getAmountFinished: piecesPercent size: pieceCount];
     
-    int i, index;
-    float increment = (float)pieceCount / MAX_PIECES;
-    NSColor * pieceColor;
-    for (i = 0; i < MAX_PIECES; i++)
+    NSBitmapImageRep * bitmap = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes: nil
+                                    pixelsWide: pieceCount pixelsHigh: 1 bitsPerSample: 8 samplesPerPixel: 4 hasAlpha: YES
+                                    isPlanar: NO colorSpaceName: NSCalibratedRGBColorSpace bytesPerRow: 0 bitsPerPixel: 0];
+    
+    NSIndexSet * previousFinishedIndexes = [torrent previousFinishedPieces];
+    NSMutableIndexSet * finishedIndexes = [NSMutableIndexSet indexSet];
+    
+    for (NSInteger i = 0; i < pieceCount; i++)
     {
-        index = i * increment;
-        if (piecePercent[index] >= 1.0)
+        NSColor * pieceColor;
+        if (piecesPercent[i] == 1.0f)
         {
-            if (previousPiecePercent != NULL && previousPiecePercent[index] < 1.0)
-                pieceColor = fOrangeColor;
+            if (previousFinishedIndexes && ![previousFinishedIndexes containsIndex: i])
+                pieceColor = [NSColor orangeColor];
             else
-                pieceColor = fBlueColor;
+                pieceColor = fBluePieceColor;
+            [finishedIndexes addIndex: i];
         }
-        else if (piecePercent[index] <= 0.0)
-            pieceColor = fGrayColor;
-        else if (piecePercent[index] <= 0.25)
-            pieceColor = fBlue1Color;
-        else if (piecePercent[index] <= 0.5)
-            pieceColor = fBlue2Color;
-        else if (piecePercent[index] <= 0.75)
-            pieceColor = fBlue3Color;
         else
-            pieceColor = fBlue4Color;
+            pieceColor = [[NSColor whiteColor] blendedColorWithFraction: piecesPercent[i] ofColor: fBluePieceColor];
         
-        if (![pieceColor isEqual: [fBitmap colorAtX: i y: 0]])
-            [fBitmap setColor: pieceColor atX: i y: 0];
+        //it's faster to just set color instead of checking previous color
+        [bitmap setColor: pieceColor atX: i y: 0];
     }
     
-    [torrent setPreviousAmountFinished: piecePercent];
+    free(piecesPercent);
+    
+    [torrent setPreviousFinishedPieces: [finishedIndexes count] > 0 ? finishedIndexes : nil]; //don't bother saving if none are complete
     
     //actually draw image
-    [fBitmap drawInRect: barRect];
-    
-    if (!fTransparentGradient)
-        fTransparentGradient = [[CTGradient progressTransparentGradient] retain];
-    [fTransparentGradient fillRect: barRect angle: -90];
+    [bitmap drawInRect: barRect];
+    [bitmap release];
 }
 
 - (NSRect) rectForMinimalStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    if (![fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
     NSRect result = bounds;
     result.size = [string size];
     
@@ -862,9 +740,6 @@
 
 - (NSRect) rectForProgressWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    if ([fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
     NSRect result = bounds;
     result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS;
     result.origin.x += PADDING_HORIZONTAL + IMAGE_SIZE_REG + PADDING_BETWEEN_IMAGE_AND_TITLE;
@@ -877,9 +752,6 @@
 
 - (NSRect) rectForStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    if ([fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
     NSRect result = bounds;
     result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS
                         + PADDING_BETWEEN_PROGRESS_AND_BAR + BAR_HEIGHT + PADDING_BETWEEN_BAR_AND_STATUS;
