@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: TorrentTableView.h 5061 2008-02-19 02:10:20Z livings124 $
+ * $Id: TorrentTableView.h 7007 2008-11-01 15:22:32Z livings124 $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -24,7 +24,7 @@
 
 #import <Cocoa/Cocoa.h>
 #import <transmission.h>
-#import <Controller.h>
+#import "Controller.h"
 
 @class TorrentCell;
 
@@ -42,30 +42,34 @@
     
     IBOutlet NSMenu * fContextRow, * fContextNoRow;
     
-    int fMouseControlRow, fMouseRevealRow, fMouseActionRow, fActionPushedRow;
+    NSInteger fMouseControlRow, fMouseRevealRow, fMouseActionRow, fActionPushedRow;
     NSArray * fSelectedValues;
     
     IBOutlet NSMenu * fActionMenu, * fUploadMenu, * fDownloadMenu, * fRatioMenu;
     Torrent * fMenuTorrent;
     
-    float fPiecesBarPercent;
-    NSTimer * fPiecesBarTimer;
+    CGFloat fPiecesBarPercent;
+    NSAnimation * fPiecesBarAnimation;
 }
 
-- (BOOL) isGroupCollapsed: (int) value;
-- (void) removeCollapsedGroup: (int) value;
+- (BOOL) isGroupCollapsed: (NSInteger) value;
+- (void) removeCollapsedGroup: (NSInteger) value;
 - (void) removeAllCollapsedGroups;
 - (void) saveCollapsedGroups;
 
 - (void) removeButtonTrackingAreas;
-- (void) setControlButtonHover: (int) row;
-- (void) setRevealButtonHover: (int) row;
-- (void) setActionButtonHover: (int) row;
+- (void) setControlButtonHover: (NSInteger) row;
+- (void) setRevealButtonHover: (NSInteger) row;
+- (void) setActionButtonHover: (NSInteger) row;
 
 - (void) selectValues: (NSArray *) values;
 - (NSArray *) selectedValues;
 - (NSArray *) selectedTorrents;
 
+- (NSRect) iconRectForRow: (NSInteger) row;
+
+- (void) paste: (id) sender;
+
 - (void) toggleControlForTorrent: (Torrent *) torrent;
 
 - (void) displayTorrentMenuForEvent: (NSEvent *) event;
@@ -77,8 +81,9 @@
 - (void) setQuickRatio: (id) sender;
 
 - (void) checkFile: (id) sender;
+- (void) moveDataFile: (id) sender;
 
 - (void) togglePiecesBar;
-- (float) piecesBarPercent;
+- (CGFloat) piecesBarPercent;
 
 @end
