@@ -15,11 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: torrent-list.cc 4494 2008-01-05 08:05:17Z charles $
+ * $Id: torrent-list.cc 6792 2008-09-19 17:17:34Z charles $
  */
 
+#include <algorithm>
 #include <wx/intl.h>
 #include <torrent-list.h>
+#include <libtransmission/utils.h>
 
 namespace
 {
@@ -144,7 +146,7 @@ namespace
         for ( i=0; size>>10; ++i )
             size = size >> 10;
         char buf[512];
-        snprintf( buf, sizeof(buf), "%.*f %s", bestDecimal(size), (double)size, sizestrs[i] );
+        tr_snprintf( buf, sizeof(buf), "%.*f %s", bestDecimal(size), (double)size, sizestrs[i] );
         return toWxStr( buf );
     }
 
@@ -236,12 +238,12 @@ TorrentListCtrl :: RefreshTorrent( tr_torrent   * tor,
         switch( *it )
         {
             case COL_POSITION:
-                snprintf( buf, sizeof(buf), "%d", 666 );
+                tr_snprintf( buf, sizeof(buf), "%d", 666 );
                 xstr = toWxStr( buf );
                 break;
 
             case COL_PERCENT_DONE:
-                snprintf( buf, sizeof(buf), "%d%%", (int)(s->percentDone*100.0) );
+                tr_snprintf( buf, sizeof(buf), "%d%%", (int)(s->percentDone*100.0) );
                 xstr = toWxStr( buf );
                 break;
 
@@ -307,7 +309,6 @@ TorrentListCtrl :: RefreshTorrent( tr_torrent   * tor,
                     case TR_STATUS_CHECK:       xstr = wxString::Format ( _("Checking Files (%.0f)"), s->recheckProgress );  break;
                     case TR_STATUS_CHECK_WAIT:  xstr = _("Waiting to Check"); break;
                     case TR_STATUS_DOWNLOAD:    xstr = _("Downloading"); break;
-                    case TR_STATUS_DONE:
                     case TR_STATUS_SEED:        xstr = _("Seeding"); break;
                     default: assert( 0 );
                 }
@@ -441,7 +442,7 @@ TorrentListCtrl :: Compare( long item1, long item2, long sortData )
 
         case COL_PEERS:
             /* FIXME: this is all peers, not just leechers 
-            snprintf( buf, sizeof(buf), "%d (%d)", s->peersTotal, s->peersConnected );
+            tr_snprintf( buf, sizeof(buf), "%d (%d)", s->peersTotal, s->peersConnected );
             xstr = toWxStr( buf );*/
             break;
 
@@ -476,7 +477,7 @@ TorrentListCtrl :: Compare( long item1, long item2, long sortData )
             break;
 
         case COL_SEEDS:
-            /*snprintf( buf, sizeof(buf), "%d", s->seeders );
+            /*tr_snprintf( buf, sizeof(buf), "%d", s->seeders );
             xstr = toWxStr( buf );*/
             break;
 
